@@ -17,7 +17,7 @@
  * along with QCAD.
  */
 
-include("../File.js");
+include("scripts/File/File.js");
 include("scripts/ImportExport/SvgImporter/SvgImporter.js");
 
 function SvgImport(guiAction) {
@@ -30,6 +30,15 @@ SvgImport.getPreferencesCategory = function() {
     return [ qsTr("File"), qsTr("SVG Import")];
 };
 
+SvgImport.initPreferences = function(pageWidget, calledByPrefDialog, document) {
+    if (!hasPlugin("PROTOOLS")) {
+        var w = pageWidget.findChild("GroupLayers");
+        if (!isNull(w)) {
+            w.visible = false;
+        }
+    }
+};
+
 SvgImport.includeBasePath = includeBasePath;
 
 SvgImport.prototype.beginEvent = function() {
@@ -57,10 +66,12 @@ SvgImport.prototype.beginEvent = function() {
         }
     }
     
+    var appWin = EAction.getMainWindow();
+
     if (isNull(fileName)) {
         var lastDir = RSettings.getStringValue( "SvgImport/Path", RSettings.getDocumentsLocation());
         fileName = QFileDialog.getOpenFileName(
-            this, qsTr("Import SVG"), lastDir,
+            appWin, qsTr("Import SVG"), lastDir,
             qsTr("SVG Files") + " (*.svg);;" + qsTr("All Files") + " (*)");
         if (fileName.length===0) {
             this.terminate();
@@ -69,12 +80,11 @@ SvgImport.prototype.beginEvent = function() {
         RSettings.setValue("SvgImport/Path", new QFileInfo(fileName).absolutePath());
     }
 
-    var appWin = EAction.getMainWindow();
     var dialog = WidgetFactory.createDialog(SvgImport.includeBasePath, "SvgImportDialog.ui", appWin);
     var resolutionCombo = dialog.findChild("Resolution");
 
     if (!dialog.exec()) {
-        dialog.destroy();
+        destr(dialog);
         EAction.activateMainWindow();
         this.terminate();
         return;
@@ -86,7 +96,7 @@ SvgImport.prototype.beginEvent = function() {
     }
 
     var svgImporter = new SvgImporter(this.getDocument(), resolution);
-    svgImporter.importFile(fileName);
+    svgImporter.importFile(fileName, "");
 
     var di = this.getDocumentInterface();
     if (!isNull(di)) {
@@ -98,7 +108,7 @@ SvgImport.prototype.beginEvent = function() {
         appWin.notifyListeners();
     }
     
-    dialog.destroy();
+    destr(dialog);
     EAction.activateMainWindow();
     this.terminate();
 };
