@@ -77,6 +77,16 @@ SvgHandler.rxTransform = new RegExp(
         "[,\\s]+" +
         "([+-]?\\d*\\.?\\d*)" +      // match 15
         "[\\s]*" +
+    "\\)" +
+    "|" +
+    "(rotate)\\s*\\(" +              // match 16
+        "\\s*" +
+        "([+-]?\\d*\\.?\\d*)" +      // match 17
+        "[,\\s]+" +
+        "([+-]?\\d*\\.?\\d*)" +      // match 18
+        "[,\\s]+" +
+        "([+-]?\\d*\\.?\\d*)" +      // match 19
+        "[\\s]*" +
     "\\)",
     "gim"
 );
@@ -97,22 +107,28 @@ SvgHandler.prototype.getTransform = function(str) {
 
         if (!isNull(match[1]) && match[1].toLowerCase()==="translate") {
             t = new QTransform();
-            t.translate(match[2], match[3]);
+            t.translate(parseFloat(match[2]), parseFloat(match[3]));
         }
         else if (!isNull(match[4]) && match[4].toLowerCase()==="rotate") {
             t = new QTransform();
-            t.rotate(match[5]);
+            t.rotate(parseFloat(match[5]));
         }
         else if (!isNull(match[6]) && match[6].toLowerCase()==="matrix") {
             t = new QTransform(
-                        match[7], match[8], 0.0,
-                        match[9], match[10], 0.0,
-                        match[11], match[12], 1.0
+                        parseFloat(match[7]), parseFloat(match[8]), 0.0,
+                        parseFloat(match[9]), parseFloat(match[10]), 0.0,
+                        parseFloat(match[11]), parseFloat(match[12]), 1.0
             );
         }
         else if (!isNull(match[13]) && match[13].toLowerCase()==="scale") {
             t = new QTransform();
-            t.scale(match[14], match[15]);
+            t.scale(parseFloat(match[14]), parseFloat(match[15]));
+        }
+        else if (!isNull(match[16]) && match[16].toLowerCase()==="rotate") {
+            t = new QTransform();
+            t.translate(parseFloat(match[18]), parseFloat(match[19]));
+            t.rotate(parseFloat(match[17]));
+            t.translate(-parseFloat(match[18]), -parseFloat(match[19]));
         }
 
         if (!isNull(t)) {
@@ -219,6 +235,7 @@ SvgHandler.prototype.startElement = function(namespaceURI, localName, qName, att
         break;
 
     case "g":
+        this.svgImporter.importGroup(atts.value("id"));
         break;
 
     case "path":
@@ -292,6 +309,10 @@ SvgHandler.prototype.endElement = function(namespaceURI, localName, qName) {
     this.displayStack.pop();
     this.elementStack.pop();
 
+    if (localName==="g") {
+        this.svgImporter.groupEnd();
+    }
+
     return true;
 };
 
@@ -311,6 +332,10 @@ function SvgImporter(document, resolution) {
         resolution = 72;
     }
 
+    if (isNull(document)) {
+        return;
+    }
+
     RFileImporterAdapter.call(this, document);
     this.setDocument(document);
     this.transform = new QTransform();
@@ -359,7 +384,7 @@ SvgImporter.prototype.importFile = function(fileName) {
     var ok = xmlReader.parse(source, false);
     file.close();
     */
-    
+
     this.endImport();
 
     return true;
@@ -378,7 +403,12 @@ SvgImporter.prototype.importShape = function(shape) {
     }
     else {
         shapeP = shape.getTransformed(this.transform);
-        shape = shapeP.data();
+        if (isFunction(shapeP.data)) {
+            shape = shapeP.data();
+        }
+        else {
+            shape = shapeP;
+        }
     }
 
     shape.scale(new RVector(1.0, -1.0));
@@ -406,9 +436,23 @@ SvgImporter.prototype.importShape = function(shape) {
     if (!isNull(this.style.stroke)) {
         entity.setColor(new RColor(this.style.stroke));
     }
+
+    this.preImportObject(entity);
+
     this.importObject(entity);
 };
 
+SvgImporter.prototype.preImportObject = function(obj) {
+};
+
+SvgImporter.prototype.importGroup = function(name) {
+    // ignore groups
+};
+
+SvgImporter.prototype.groupEnd = function() {
+    // ignore groups
+};
+
 SvgImporter.prototype.importCircle = function(x, y, r) {
     this.importShape(new RCircle(new RVector(x, y), r));
 };
@@ -424,28 +468,33 @@ SvgImporter.prototype.importRectangle = function(x, y, w, h) {
     this.importShape(new RLine(new RVector(x, y+h), new RVector(x, y)));
 };
 
-SvgImporter.prototype.importLine = function(x1, y1, x2, y2) {
-    this.importShape(new RLine(new RVector(x1, y1), new RVector(x2, y2)));
+SvgImporter.prototype.createLine = function(x1, y1, x2, y2) {
+    return new RLine(new RVector(x1, y1), new RVector(x2, y2));
 };
 
-SvgImporter.prototype.importArc = function(ox, oy, x, y, rx, ry, angle, isLarge, sweep) {
+SvgImporter.prototype.importLine = function(x1, y1, x2, y2) {
+    this.importShape(this.createLine(x1, y1, x2, y2));
+};
+
+SvgImporter.prototype.createArc = function(ox, oy, x, y, rx, ry, angle, isLarge, sweep) {
     var entity;
     var center = SvgImporter.getArcCenter(ox, oy, x, y, rx, ry, sweep, !isLarge, angle);
     if (Math.abs(rx-ry) < RS.PointTolerance) {
-        this.importShape(new RArc(center, rx, center.getAngleTo(new RVector(ox, oy)), center.getAngleTo(new RVector(x, y)), !sweep));
+        return new RArc(center, rx, center.getAngleTo(new RVector(ox, oy)), center.getAngleTo(new RVector(x, y)), !sweep);
     }
     else {
         // TODO
         //var ellipse = new REllipseArc();
         //entity = new REllipseEntity(this.getDocument(), new REllipseData(ellipse));
     }
+    return undefined;
 };
 
 /**
  * Imports a cubic spline with the given start point, control points and ent point.
  * If the spline is almost exactly resembling an arc, an arc is imported instead.
  */
-SvgImporter.prototype.importBezier = function(x1, y1, px1, py1, px2, py2, x2, y2) {
+SvgImporter.prototype.createBezier = function(x1, y1, px1, py1, px2, py2, x2, y2) {
     var spline = new RSpline();
     spline.setDegree(3);
     spline.appendControlPoint(new RVector(x1, y1));
@@ -462,16 +511,18 @@ SvgImporter.prototype.importBezier = function(x1, y1, px1, py1, px2, py2, x2, y2
 //    }
 //    this.importShape(shape);
 
-    this.importShape(spline);
+    return spline;
+    //this.importShape(spline);
 };
 
-SvgImporter.prototype.importBezier2 = function(x1, y1, px, py, x2, y2) {
+SvgImporter.prototype.createBezier2 = function(x1, y1, px, py, x2, y2) {
     var spline = new RSpline();
     spline.setDegree(2);
     spline.appendControlPoint(new RVector(x1, y1));
     spline.appendControlPoint(new RVector(px, py));
     spline.appendControlPoint(new RVector(x2, y2));
-    this.importShape(spline);
+    return spline;
+    //this.importShape(spline);
 };
 
 /**
@@ -479,7 +530,14 @@ SvgImporter.prototype.importBezier2 = function(x1, y1, px, py, x2, y2) {
  * from the path.
  */
 SvgImporter.prototype.importPath = function(dData) {
-    var segs = dData.split(/(?=[mlhvcsqtaz])/i);
+    var segs;
+    if (RSettings.getQtVersion()<0x060000) {
+        segs = dData.split(/(?=[mlhvcsqtaz])/i);
+    }
+    else {
+        segs = dData.split(/([mlhvcsqtaz][^mlhvcsqtaz]*)/i);
+    }
+
     var x = 0;
     var y = 0;
     var x0, y0;  // start point of sub path for z command
@@ -493,12 +551,19 @@ SvgImporter.prototype.importPath = function(dData) {
     var isLarge;
     var sweep;
 
+    var shapes = [];
+
+    var rxLeadingZero = new RegExp(" 0([0-9])", "g");
     for (var i=0; i<segs.length; i++) {
         var seg = segs[i];
         if (seg.trim().length===0) {
             continue;
         }
 
+        while (seg.match(rxLeadingZero)) {
+            seg = seg.replace(rxLeadingZero, " 0 $1");
+        }
+
         var cmd = seg.match(/[mlhvcsqtaz]/i);
         if (!cmd) {
             //qDebug("abort...");
@@ -522,9 +587,11 @@ SvgImporter.prototype.importPath = function(dData) {
                 px = py = null;
                 // coordinates after the first: implicit line to:
                 if (k!==0) {
-                    this.importLine(ox, oy, x, y);
+                    shapes.push(new RLine(ox, oy, x, y));
+                    //this.importLine(ox, oy, x, y);
                 }
                 else {
+                    shapes.push(undefined);
                     x0 = x;
                     y0 = y;
                 }
@@ -541,9 +608,11 @@ SvgImporter.prototype.importPath = function(dData) {
                 px = py = null;
                 // coordinates after the first: implicit line to:
                 if (k!==0) {
-                    this.importLine(ox, oy, x, y);
+                    shapes.push(new RLine(ox, oy, x, y));
+                    //this.importLine(ox, oy, x, y);
                 }
                 else {
+                    shapes.push(undefined);
                     x0 = x;
                     y0 = y;
                 }
@@ -558,7 +627,8 @@ SvgImporter.prototype.importPath = function(dData) {
                 x = coords[k+0];
                 y = coords[k+1];
                 px = py = null;
-                this.importLine(ox, oy, x, y);
+                shapes.push(new RLine(ox, oy, x, y));
+                //this.importLine(ox, oy, x, y);
                 ox = x;
                 oy = y;
             }
@@ -569,7 +639,8 @@ SvgImporter.prototype.importPath = function(dData) {
                 x += coords[k+0];
                 y += coords[k+1];
                 px = py = null;
-                this.importLine(ox, oy, x, y);
+                shapes.push(new RLine(ox, oy, x, y));
+                //this.importLine(ox, oy, x, y);
                 ox = x;
                 oy = y;
             }
@@ -579,7 +650,8 @@ SvgImporter.prototype.importPath = function(dData) {
             for (k=0; k<coords.length; k+=1) {
                 x = coords[k+0];
                 px = py = null;
-                this.importLine(ox, oy, x, y);
+                shapes.push(new RLine(ox, oy, x, y));
+                //this.importLine(ox, oy, x, y);
                 ox = x;
             }
             break;
@@ -588,7 +660,8 @@ SvgImporter.prototype.importPath = function(dData) {
             for (k=0; k<coords.length; k+=1) {
                 x += coords[k+0];
                 px = py = null;
-                this.importLine(ox, oy, x, y);
+                shapes.push(new RLine(ox, oy, x, y));
+                //this.importLine(ox, oy, x, y);
                 ox = x;
             }
             break;
@@ -597,7 +670,8 @@ SvgImporter.prototype.importPath = function(dData) {
             for (k=0; k<coords.length; k+=1) {
                 y = coords[k+0];
                 px = py = null;
-                this.importLine(ox, oy, x, y);
+                shapes.push(new RLine(ox, oy, x, y));
+                //this.importLine(ox, oy, x, y);
                 oy = y;
             }
             break;
@@ -606,7 +680,8 @@ SvgImporter.prototype.importPath = function(dData) {
             for (k=0; k<coords.length; k+=1) {
                 y += coords[k+0];
                 px = py = null;
-                this.importLine(ox, oy, x, y);
+                shapes.push(new RLine(ox, oy, x, y));
+                //this.importLine(ox, oy, x, y);
                 oy = y;
             }
             break;
@@ -617,10 +692,12 @@ SvgImporter.prototype.importPath = function(dData) {
                 y = coords[k+5];
                 px = coords[k+2];
                 py = coords[k+3];
-                this.importBezier(ox, oy,
-                                  coords[k+0], coords[k+1],
-                                  coords[k+2], coords[k+3],
-                                  coords[k+4], coords[k+5]);
+                shapes.push(
+                            this.createBezier(ox, oy,
+                                              coords[k+0], coords[k+1],
+                                              coords[k+2], coords[k+3],
+                                              coords[k+4], coords[k+5])
+                            );
                 ox = x;
                 oy = y;
             }
@@ -633,10 +710,14 @@ SvgImporter.prototype.importPath = function(dData) {
 //                       coords[k+0] + x, coords[k+1] + y,
 //                       coords[k+2] + x, coords[k+3] + y,
 //                       coords[k+4] + x, coords[k+5] + y].join(" | "));
-                this.importBezier(ox, oy,
-                                  coords[k+0] + x, coords[k+1] + y,
-                                  coords[k+2] + x, coords[k+3] + y,
-                                  coords[k+4] + x, coords[k+5] + y);
+
+                shapes.push(
+                            this.createBezier(ox, oy,
+                                              coords[k+0] + x, coords[k+1] + y,
+                                              coords[k+2] + x, coords[k+3] + y,
+                                              coords[k+4] + x, coords[k+5] + y)
+                            );
+
                 px = x + coords[k+2];
                 py = y + coords[k+3];
                 x += coords[k+4];
@@ -651,10 +732,14 @@ SvgImporter.prototype.importPath = function(dData) {
                 px = x;
                 py = y;
             }
-            this.importBezier(ox, oy,
-                              x-(px-x), y-(py-y),
-                              coords[0], coords[1],
-                              coords[2], coords[3]);
+
+            shapes.push(
+                        this.createBezier(ox, oy,
+                                          x-(px-x), y-(py-y),
+                                          coords[0], coords[1],
+                                          coords[2], coords[3])
+                        );
+
             px = coords[0];
             py = coords[1];
             x = coords[2];
@@ -666,10 +751,15 @@ SvgImporter.prototype.importPath = function(dData) {
                 px = x;
                 py = y;
             }
-            this.importBezier(ox, oy,
-                              x-(px-x), y-(py-y),
-                              x + coords[0], y + coords[1],
-                              x + coords[2], y + coords[3]);
+
+            shapes.push(
+                        this.createBezier(ox, oy,
+                                          x-(px-x), y-(py-y),
+                                          x + coords[0], y + coords[1],
+                                          x + coords[2], y + coords[3])
+                        );
+
+
             px = x + coords[0];
             py = y + coords[1];
             x += coords[2];
@@ -681,15 +771,19 @@ SvgImporter.prototype.importPath = function(dData) {
             py = coords[1];
             x = coords[2];
             y = coords[3];
-            this.importBezier2(ox, oy,
+            shapes.push(
+                        this.createBezier2(ox, oy,
                               coords[0], coords[1],
-                              coords[2], coords[3]);
+                              coords[2], coords[3])
+                        );
             break;
 
         case 'q':
-            this.importBezier2(ox, oy,
-                              coords[0] + x, coords[1] + y,
-                              coords[2] + x, coords[3] + y);
+            shapes.push(
+                        this.createBezier2(ox, oy,
+                                           coords[0] + x, coords[1] + y,
+                                           coords[2] + x, coords[3] + y)
+                        );
             px = x + coords[0];
             py = y + coords[1];
             x += coords[2];
@@ -704,9 +798,11 @@ SvgImporter.prototype.importPath = function(dData) {
                 px = x-(px-x);
                 py = y-(py-y);
             }
-            this.importBezier2(ox, oy,
-                              px, py,
-                              coords[0], coords[1]);
+            shapes.push(
+                        this.createBezier2(ox, oy,
+                                           px, py,
+                                           coords[0], coords[1])
+                        );
             px = x-(px-x);
             py = y-(py-y);
             x = coords[0];
@@ -721,9 +817,11 @@ SvgImporter.prototype.importPath = function(dData) {
                 px = x-(px-x);
                 py = y-(py-y);
             }
-            this.importBezier2(ox, oy,
-                              px, py,
-                              coords[0] + x, coords[1] + y);
+            shapes.push(
+                        this.createBezier2(ox, oy,
+                                           px, py,
+                                           coords[0] + x, coords[1] + y)
+                        );
             x += coords[0];
             y += coords[1];
             break;
@@ -738,7 +836,9 @@ SvgImporter.prototype.importPath = function(dData) {
                 x = coords[k+5];
                 y = coords[k+6];
                 px = py = null;
-                this.importArc(ox, oy, x, y, rx, ry, angle, isLarge===1, sweep===1);
+                shapes.push(
+                            this.createArc(ox, oy, x, y, rx, ry, angle, isLarge===1, sweep===1)
+                            );
                 ox = x;
                 oy = y;
             }
@@ -754,7 +854,10 @@ SvgImporter.prototype.importPath = function(dData) {
                 x += coords[k+5];
                 y += coords[k+6];
                 px = py = null;
-                this.importArc(ox, oy, x, y, rx, ry, angle, isLarge===1, sweep===1);
+                shapes.push(
+                            this.createArc(ox, oy, x, y, rx, ry, angle, isLarge===1, sweep===1)
+                            );
+
                 ox = x;
                 oy = y;
             }
@@ -765,7 +868,7 @@ SvgImporter.prototype.importPath = function(dData) {
             x = x0;
             y = y0;
             if (ox!==x || oy!==y) {
-                this.importLine(ox,oy, x,y);
+                shapes.push(new RLine(ox,oy, x,y));
             }
             ox = x;
             oy = y;
@@ -773,6 +876,35 @@ SvgImporter.prototype.importPath = function(dData) {
         }
         pc = cmd;
     }
+
+    // add as many shapes as polyline as possible:
+    var pl = new RPolyline();
+    for (var i=0; i<shapes.length; i++) {
+        var shape = shapes[i];
+
+        if (isLineShape(shape) || isArcShape(shape)) {
+            pl.appendShape(shape);
+        }
+        else {
+            // shape cannot be part of a polyline or is undefined (gap):
+
+            // end current polyline:
+            if (!pl.isEmpty()) {
+                this.importShape(pl);
+            }
+            // start new polyline:
+            pl = new RPolyline();
+
+            // import shape that is not a line or arc separately:
+            if (!isNull(shape)) {
+                this.importShape(shape);
+            }
+        }
+    }
+    if (!pl.isEmpty()) {
+        this.importShape(pl);
+    }
+
 };
 
 /**
