@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2018 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2021 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -33,6 +33,7 @@ function Scale(guiAction) {
     this.targetPoint = undefined;
     this.factorX = undefined;
     this.factorY = undefined;
+    this.factorByMouse = false;
 
     this.useDialog = RSettings.getBoolValue("Scale/UseDialog", true);
 
@@ -49,7 +50,7 @@ Scale.getPreferencesCategory = function() {
 };
 
 Scale.State = {
-    SettingCenterPoint : 0,
+    SettingFocusPoint : 0,
     SettingReferencePoint : 1,
     SettingTargetPoint : 2
 };
@@ -61,7 +62,7 @@ Scale.prototype.beginEvent = function() {
         return;
     }
 
-    this.setState(Scale.State.SettingCenterPoint);
+    this.setState(Scale.State.SettingFocusPoint);
 };
 
 Scale.prototype.setState = function(state) {
@@ -72,7 +73,7 @@ Scale.prototype.setState = function(state) {
 
     var appWin = RMainWindowQt.getMainWindow();
     switch (this.state) {
-    case Scale.State.SettingCenterPoint:
+    case Scale.State.SettingFocusPoint:
         this.referencePoint = undefined;
         this.targetPoint = undefined;
         //this.factorX = undefined;
@@ -116,12 +117,12 @@ Scale.prototype.initUiOptions = function(resume, optionsToolBar) {
 
 Scale.prototype.escapeEvent = function() {
     switch (this.state) {
-    case Scale.State.SettingCenterPoint:
+    case Scale.State.SettingFocusPoint:
         Transform.prototype.escapeEvent.call(this);
         break;
 
     case Scale.State.SettingReferencePoint:
-        this.setState(Scale.State.SettingCenterPoint);
+        this.setState(Scale.State.SettingFocusPoint);
         break;
 
     case Scale.State.SettingTargetPoint:
@@ -135,7 +136,7 @@ Scale.prototype.pickCoordinate = function(event, preview) {
     var op;
 
     switch (this.state) {
-    case Scale.State.SettingCenterPoint:
+    case Scale.State.SettingFocusPoint:
         this.focusPoint = event.getModelPosition();
 
         if (preview) {
@@ -161,10 +162,15 @@ Scale.prototype.pickCoordinate = function(event, preview) {
                 op = this.getOperation(false);
                 if (!isNull(op)) {
                     di.applyOperation(op);
-                    di.setRelativeZero(this.focusPoint);
+                    this.terminate();
+                }
+                else {
+                    EAction.handleUserWarning(qsTr("Invalid scale factor"));
                     this.terminate();
                 }
             }
+
+            di.setRelativeZero(this.focusPoint);
         }
         break;
 
@@ -213,7 +219,7 @@ Scale.prototype.showDialog = function() {
     WidgetFactory.restoreState(dialog);
 
     if (!dialog.exec()) {
-        dialog.destroy();
+        destr(dialog);
         EAction.activateMainWindow();
         return false;
     }
@@ -248,7 +254,7 @@ Scale.prototype.showDialog = function() {
 
     WidgetFactory.saveState(dialog);
 
-    dialog.destroy();
+    destr(dialog);
     EAction.activateMainWindow();
     return true;
 };
@@ -313,54 +319,123 @@ Scale.prototype.getOperation = function(preview) {
     return Transform.prototype.getOperation.call(this, preview);
 };
 
+Scale.prototype.transformArc = function(shape, sv) {
+    var s = shape;
+    if (isFunction(shape.data)) {
+        s = getPtr(shape);
+    }
+
+    if (isCircleShape(s)) {
+        s = ShapeAlgorithms.circleToArc(s, 0.0);
+    }
+
+    return RShape.scaleArc(s, sv, this.focusPoint);
+};
+
 /**
  * Callback function for Transform.getOperation.
  */
 Scale.prototype.transform = function(entity, k, op, preview, flags) {
     // uniform scaling (supported by all entities):
-    if (isNull(this.factorY)) {
+    if (isNull(this.factorY) || RMath.fuzzyCompare(this.factorX, this.factorY)) {
         entity.scale(Math.pow(this.factorX, k), this.focusPoint);
         op.addObject(entity, flags);
         return;
     }
 
+    this.preTransform(entity);
+
+    var e;
+
     // non-uniform scaling of arc, circle or ellipse:
     var sv = new RVector(Math.pow(this.factorX, k), Math.pow(this.factorY, k));
+
     if (isArcEntity(entity) || isCircleEntity(entity) || isEllipseEntity(entity)) {
         var arc = entity.castToShape();
-        if (isCircleShape(arc)) {
-            arc = ShapeAlgorithms.circleToArc(arc);
-        }
 
-        var self=this;
-        var shape = ShapeAlgorithms.transformArc(
-            arc,
-            function(p) {
-                return p.scale(sv, self.focusPoint);
-            }
-        );
+        var shape = this.transformArc(arc, sv);
 
-        var e = shapeToEntity(this.getDocument(), shape);
+        e = shapeToEntity(this.getDocument(), shape);
         if (!isNull(e)) {
             e.copyAttributesFrom(entity);
             e.setDrawOrder(entity.getDrawOrder());
             e.setSelected(true);
         }
 
-        if (this.copies===0) {
+        if (this.getCopies()===0) {
             op.deleteObject(entity);
         }
         if (!isNull(e)) {
+            this.postTransform(e);
             op.addObject(e, flags);
         }
         return;
     }
 
+    // non-uniform scaling of hatches:
+    if (isHatchEntity(entity)) {
+        //debugger;
+        var data = entity.getData();
+        var newHatchData = new RHatchData();
+        newHatchData.setDocument(data.getDocument());
+        newHatchData.copyAttributesFrom(data);
+
+        //newHatchData.clearBoundary();
+
+        for (var i=0; i<entity.getLoopCount(); i++) {
+            newHatchData.newLoop();
+
+            var shapes = entity.getLoopBoundary(i);
+            for (var n=0; n<shapes.length; n++) {
+                shape = shapes[n];
+
+                if (isArcShape(shape) || isCircleShape(shape) || isEllipseShape(shape)) {
+                    var newShape = this.transformArc(shape, sv);
+                    if (RSettings.getQtVersion() >= 0x060000) {
+                        newHatchData.addBoundary(newShape.clone());
+                    }
+                    else {
+                        newHatchData.addBoundary(newShape);
+                    }
+                }
+                else {
+                    // line, spline:
+                    shape.scale(sv, this.focusPoint);
+                    if (RSettings.getQtVersion() >= 0x060000) {
+                        newHatchData.addBoundary(shape.clone());
+                    }
+                    else {
+                        newHatchData.addBoundary(shape);
+                    }
+                }
+            }
+        }
+        newHatchData.setSolid(entity.isSolid());
+        newHatchData.setPatternName(entity.getPatternName());
+        newHatchData.setScale(entity.getScale());
+        newHatchData.setAngle(entity.getAngle());
+        newHatchData.setOriginPoint(entity.getOriginPoint());
+
+        entity.setData(newHatchData);
+        this.postTransform(entity);
+        op.addObject(entity, flags);
+        return;
+    }
+
     // non-uniform scaling of other entities:
     entity.scale(sv, this.focusPoint);
+    this.postTransform(entity);
     op.addObject(entity, flags);
 };
 
+Scale.prototype.preTransform = function(entity) {
+    return;
+};
+
+Scale.prototype.postTransform = function(entity) {
+    return;
+};
+
 Scale.prototype.getAuxPreview = function() {
     if (!isValidVector(this.focusPoint)) {
         return undefined;
