@@ -75,7 +75,7 @@ PropertyWatcher.prototype.propertyChanged = function(value) {
             if (this.sender.itemData(value)===PropertyEditor.varies) {
                 return;
             }
-            this.propertyEditor.propertyChanged(this.propertyType,
+            this.propertyEditor.propertyChanged(this.propertyType, 
                                                 this.sender.itemData(value));
             return;
         }
@@ -1338,6 +1338,7 @@ PropertyEditorImpl.prototype.initChoiceControls = function(
                 var pw = new PropertyWatcher(this, control, propertyTypeId);
                 if (RSettings.getQtVersion() >= 0x060000) {
                     control.textActivated.connect(pw, pw.propertyChanged);
+                    //control.currentTextChanged.connect(pw, pw.propertyChanged);
                 }
                 else {
                     control['activated(QString)'].connect(pw, pw.propertyChanged);
@@ -1347,6 +1348,7 @@ PropertyEditorImpl.prototype.initChoiceControls = function(
                 var pw = new PropertyWatcher(this, control, propertyTypeId);
                 if (RSettings.getQtVersion() >= 0x060000) {
                     control.activated.connect(pw, pw.propertyChanged);
+                    //control.currentIndexChanged.connect(pw, pw.propertyChanged);
                 }
                 else {
                     control['activated(int)'].connect(pw, pw.propertyChanged);
@@ -1356,14 +1358,20 @@ PropertyEditorImpl.prototype.initChoiceControls = function(
 
     }
 
+    //var signalsBlocked;
+
     if (propertyTypeId.getId()===REntity.PropertyLayer.getId()) {
         WidgetFactory.initLayerCombo(control, EAction.getDocument());
     }
     else if (propertyTypeId.getId()===RTextEntity.PropertyHAlign.getId()) {
+        //signalsBlocked = control.blockSignals(true);
         WidgetFactory.initHAlignCombo(control);
+        //control.blockSignals(signalsBlocked);
     }
     else if (propertyTypeId.getId()===RTextEntity.PropertyVAlign.getId()) {
+        //signalsBlocked = control.blockSignals(true);
         WidgetFactory.initVAlignCombo(control);
+        //control.blockSignals(signalsBlocked);
     }
     else if (propertyTypeId.getId()===RDimensionEntity.PropertyDimlunit.getId()) {
         WidgetFactory.initDimlunitCombo(control);
@@ -1530,7 +1538,7 @@ PropertyEditorImpl.prototype.filterChanged = function() {
         entityTypeFilter = RS.EntityAll;
     }
 
-    this.updateFromDocument(doc, false, entityTypeFilter, true);
+    this.updateFromDocumentNow(doc, false, entityTypeFilter, true);
 };
 
 /**
@@ -1548,7 +1556,7 @@ PropertyEditorImpl.prototype.requestAllProperties = function() {
         entityTypeFilter = RS.EntityAll;
     }
 
-    this.updateFromDocument(doc, false, entityTypeFilter, true, true);
+    this.updateFromDocumentNow(doc, false, entityTypeFilter, true, true);
 };
 
 PropertyEditorImpl.prototype.getAdjustedPropertyValue = function(propertyTypeId) {
@@ -1634,8 +1642,7 @@ PropertyEditorImpl.prototype.addCustomProperty = function() {
     buttonBox.button(QDialogButtonBox.Ok).enabled = false;
 
     var nameEdit = dialog.findChild("Name");
-    var rx = new RegExp("[^<>\\\\\":;\?\*|,=`]{1,255}");
-    var validator = new QRegExpValidator(rx, nameEdit);
+    var validator = createValidator("[^<>\\\\\":;\?\*|,=`]{1,255}", nameEdit);
     nameEdit.setValidator(validator);
     nameEdit.textChanged.connect(
         function(text) {
@@ -1688,6 +1695,18 @@ PropertyEditor.initPreferences = function(pageWidget, calledByPrefDialog, docume
     w.addItem("7 (1/128)", 7);
     w.addItem("8 (1/256)", 8);
     w.currentIndex = 8;
+
+    // hide Qt 6 only options:
+    if (RSettings.getQtVersion() < 0x060000) {
+        var widgets = ["LazyUpdate", "LazyUpdateDelay_Label", "LazyUpdateDelay"];
+        for (var i=0; i<widgets.length; i++) {
+            w = pageWidget.findChild(widgets[i]);
+            if (!isNull(w)) {
+                w.visible = false;
+            }
+        }
+
+    }
 };
 
 PropertyEditor.applyPreferences = function(doc, mdiChild) {
